/*
    Supported chipset list.
    Copyright (C) 2011 2012 2014 2015 2016  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __CHIPSET_C
#include "chipset.h"				// functions prototypes

#include <stdio.h>				// NULL
#include <string.h>				// strcmp

#include "hd44780.h"
#include "at24c02a.h"
#include "ssd1328.h"
#include "pcf8833.h"
#include "s1d15g10.h"
#include "pscard.h"

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* macro interne alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

static const DrvInt_t		ChipsetList[]={
  CHIP_SET(	"HD44780",		// nome del driver
  		"hd44780.c",		// nome del file che lo implementa
		HD44780_LinkMap,	// mappa collegamenti
		HD44780_InitBus,
		HD44780_CmdWrite,
		HD44780_TextWrite,
		NULL,
		NULL,
		NULL,
		0,			// block size
		0),			// num of blocks
  CHIP_SET(	"AT24C02A",
  		"at24c02a.c",
		AT24C02A_LinkMap,
		AT24C02A_InitBus,
		NULL,
		AT24C02A_TextWrite,
		AT24C02A_TextRead,
		AT24C02A_BlockWrite,
		AT24C02A_BlockRead,
		8,			// block size
		32),			// num of blocks
  CHIP_SET(	"AT24C04A",
  		"at24c02a.c",
		AT24C02A_LinkMap,
		AT24C02A_InitBus,
		NULL,
		AT24C02A_TextWrite,
		AT24C02A_TextRead,
		AT24C02A_BlockWrite,
		AT24C02A_BlockRead,
		16,			// block size
		32),			// num of blocks
  CHIP_SET(	"AT24C08A",
  		"at24c02a.c",
		AT24C02A_LinkMap,
		AT24C02A_InitBus,
		NULL,
		AT24C02A_TextWrite,
		AT24C02A_TextRead,
		AT24C02A_BlockWrite,
		AT24C02A_BlockRead,
		16,			// block size
		64),			// num of blocks
  CHIP_SET(	"AT24C16A",
  		"at24c02a.c",
		AT24C02A_LinkMap,
		AT24C02A_InitBus,
		NULL,
		AT24C02A_TextWrite,
		AT24C02A_TextRead,
		AT24C02A_BlockWrite,
		AT24C02A_BlockRead,
		16,			// block size
		128),			// num of blocks
  CHIP_SET(	"SSD1328",
  		"ssd1328.c",
		SSD1328_LinkMap,
		SSD1328_InitBus,
		SSD1328_CmdWrite,
		SSD1328_TextWrite,
		NULL,
		SSD1328_CopyFrame,
		NULL,
		64,			// block size
		128),			// num of blocks
  CHIP_SET(	"PCF8833",
  		"pcf8833.c",
		PCF8833_LinkMap,
		PCF8833_InitBus,
		PCF8833_CmdWrite,
		PCF8833_TextWrite,
		NULL,
		PCF8833_CopyFrame,
		NULL,
		64,			// block size
		128),			// num of blocks
  CHIP_SET(	"S1D15G10",
  		"s1d15g10.c",
		S1D15G10_LinkMap,
		S1D15G10_InitBus,
		S1D15G10_CmdWrite,
		S1D15G10_TextWrite,
		NULL,
		S1D15G10_CopyFrame,
		NULL,
		64,			// block size
		128),			// num of blocks
  CHIP_SET(	"PSCARD",
  		"pscard.c",
		PSCARD_LinkMap,
		PSCARD_InitBus,
		NULL,
		NULL,
		NULL,
		PSCARD_BlockWrite,
		PSCARD_BlockRead,
		128,			// block size
		32),			// num of blocks
  END_OF_LIST()};

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

const DrvInt_t *ChipsetSearch(const char *DrvName)
{
  /* Cerca un driver con il nome indicato e ritorna un puntatore al driver
     stesso, ritorna NULL se non esiste un driver col nome indicato.
     Chiamata con DrvName=NULL ritorna il primo chipset della lista. */
  
  
  
  const DrvInt_t	*DrvPtr=ChipsetList;	// puntatore al driver



  if(DrvName == NULL)				// se chiama con NULL
    return(ChipsetList);			// ritorna il primo della lista
  while(!IsEndOfList(DrvPtr))			// se la lista non e' finita
  {
    if(strcmp(DrvPtr->DrvName,DrvName) == 0)	// se trova il driver cercato
      return(DrvPtr);				// termina con successo
    DrvPtr++;					// passa al driver successivo
  }
  return(NULL);					// termina senza aver trovato
}



int ChipsetMaxBlock(void)
{
  /* Ritorna la dimensione del singolo blocco piu' grande tra tutti i
     chipset supportati. */
  
  
  
  const DrvInt_t	*DrvPtr=ChipsetList;	// puntatore al driver
  int	Max=0;



  while(!IsEndOfList(DrvPtr))			// fino a fine lista
  {
    if(DrvPtr->BlockSize > Max)			// se piu' del massimo
      Max=DrvPtr->BlockSize;			// aggiorna il massimo
    DrvPtr++;					// next chipset
  }
  return(Max);					// termina
}



int ChipsetMaxSize(void)
{
  /* Ritorna la dimensione totale piu' grande tra tutti i chipset supportati. */
  
  
  
  const DrvInt_t	*DrvPtr=ChipsetList;	// puntatore al driver
  int	Max=0;
  int	Total;



  while(!IsEndOfList(DrvPtr))			// fino a fine lista
  {
    Total=DrvPtr->BlockSize * DrvPtr->BlocksNum;
    if(Total > Max)				// se piu' del massimo
      Max=Total;				// aggiorna il massimo
    DrvPtr++;					// next chipset
  }
  return(Max);					// termina
}

/* $Source$ */

