/*
    PCF8833 driver implementation.
    Copyright (C) 2011  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __PCF8833_C
#include "pcf8833.h"				// functions prototypes

#include "spi.h"				// SPI bus handling functions
#include <stdio.h>				// NULL
#include <unistd.h>				// sleep

/***************************************************************************/
/* define interni alla libreria */

#define BUFFER_SIZE	64*128			// dimensione frame buffer

// hardware pins link
#define MOSI		0x04
#define MISO		0x10
#define CLK		0x01
#define CS		0x02
#define RES		0x08
#define NEN		0x40
#define SEL		0x80
#define DC		0x20

// display commands
#define NOP		0x00,0,NULL
#define SWRESET		0x01,0,NULL
#define BSTROFF		0x02,0,NULL
#define BSTRON		0x03,0,NULL
#define SLPIN		0x10,0,NULL
#define SLPOUTP		0x11,0,NULL
#define PTLON		0x12,0,NULL
#define NORON		0x13,0,NULL
#define INVOFF		0x20,0,NULL
#define INVON		0x21,0,NULL
#define APOFF		0x22,0,NULL
#define APON		0x23,0,NULL
#define SETCON		0x25,1
#define DISPOFF		0x28,0,NULL
#define DISPON		0x29,0,NULL
#define CASET		0x2a,2
#define PASET		0x2b,2
#define RAMWR		0x2c
#define RGBSET		0x2d,20
#define PTLAR		0x30,2
#define VSCRDEF		0x33,3
#define TEOFF		0x34,0,NULL
#define TEON		0x35,1
#define MADCTL		0x36,1
#define SEP		0x37,1
#define IDMOFF		0x38,0,NULL
#define IDMON		0x39,0,NULL
#define COLMOD		0x3a,1
#define SETVOP		0xb0,2
#define BSR		0xb4,0,NULL
#define TSR		0xb6,0,NULL
#define FINV		0xb9,0,NULL
#define DOR		0xba,0,NULL
#define TCDFE		0xbd,0,NULL
#define TCVOPE		0xbf,0,NULL
#define EC		0xc0,0,NULL
#define SETMUL		0xc2,1
#define TCVOPAB		0xc3,1
#define TCVOPCD		0xc4,1
#define TCDF		0xc5,4
#define DF8		0xc6,1
#define SETBS		0xc7,0,NULL
#define NLI		0xc9,1

/***************************************************************************/
/* macro interne alla libreria */

#define SetCS(V)	SPI_SetSpare(0,V)
#define SetRES(V)	SPI_SetSpare(1,V)

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

static const unsigned char	Signal[]={MOSI,MISO,CLK,CS,RES,NEN,SEL,DC};

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

int PCF8833_InitBus(void)
{
  /* Imposta lo stato iniziale e la direzione delle linee del bus e
     inizializza il display driver. */



  unsigned char	Data[24];			// command data



  // inizializzail bus
//  if(SPI_InitBus(SPI_CPOL_HI|SPI_SECOND_EDGE|SPI_MSB_FIRST,Signal))
  if(SPI_InitBus(SPI_CPOL_LOW|SPI_FIRST_EDGE|SPI_MSB_FIRST,Signal))
    return 1;					// termina con fallimento
  SetRES(0);					// resetta il driver
  SetCS(1);					// alza CS
  sleep(1);					// pausa
  SetRES(1);					// rilascia il reset

  // display init
  Data[0]=	0x00;
  Data[1]=	0x20;
  Data[2]=	0x0a;
  Data[3]=	0x01;
  S1D15G10_SendCommand(DISCTL,Data);
  Data[0]=	0x01;
  S1D15G10_SendCommand(COMSCN,Data);
  S1D15G10_SendCommand(OSCON);
  S1D15G10_SendCommand(SLPOUT);
  S1D15G10_SetContrast(28,3);			// imposta il contrasto
//  Data[0]=	0x00;
//  S1D15G10_SendCommand(TMPGRD,Data);
  Data[0]=	0x0f;
  S1D15G10_SendCommand(PWRCTR,Data);
  S1D15G10_SendCommand(DISINV);
  S1D15G10_SendCommand(PTLOUT);			// abbandona partial display
  Data[0]=	0x00;
  Data[1]=	0x00;
  Data[2]=	0x01;				// 8 bit grayscale
  S1D15G10_SendCommand(DATCTL,Data);
  Data[0]=	0x00;
  Data[1]=	0x02;
  Data[2]=	0x04;
  Data[3]=	0x06;
  Data[4]=	0x08;
  Data[5]=	0x0a;
  Data[6]=	0x0c;
  Data[7]=	0x0f;
  Data[8]=	0x00;
  Data[9]=	0x02;
  Data[10]=	0x04;
  Data[11]=	0x06;
  Data[12]=	0x08;
  Data[13]=	0x0a;
  Data[14]=	0x0c;
  Data[15]=	0x0f;
  Data[16]=	0x00;
  Data[17]=	0x06;
  Data[18]=	0x09;
  Data[19]=	0x0f;
  S1D15G10_SendCommand(RGBSET8,Data);
  S1D15G10_SendCommand(DISON);
  return 0;					// termina con successo
}



void PCF8833_SendCommand(unsigned char Cmd,int DataLen,unsigned char Data[])
{
  /* Invia un byte comando seguito da un array di argomenti a display. */



  int	i;
  
  
  
  printf("%s: 0x%02x",__FUNCTION__,(unsigned int)Cmd);
  SetCS(0);						// abbassa CS
  SPI_DataExchange(9,Cmd);				// invia il comando
  SetCS(1);						// alza CS
  for(i=0;i<DataLen;i++)
  {
    printf(" 0x%02x",(unsigned int)Data[i]);
    SetCS(0);						// abbassa CS
    SPI_DataExchange(9,0x0100 | (unsigned long int)Data[i]);	// dato
    SetCS(1);						// alza CS
  }
  puts("");						// va a capo
  return;						// termina
}



void PCF8833_SetWindow(unsigned char x1,unsigned char y1,
			unsigned char x2,unsigned char y2)
{
  /* Imposta l'area accessibile in scrittura sul display. */
  
  
  
  unsigned char	Data[2];			// command data



  Data[0]=	y1;				// start page
  Data[1]=	y2;				// end page
  S1D15G10_SendCommand(PASET,Data);
  Data[0]=	x1;				// start column
  Data[1]=	x2;				// end column
  S1D15G10_SendCommand(CASET,Data);
  return;					// termina
}



void PCF8833_SetContrast(unsigned char Volume,unsigned char Res)
{
  /* Imposta i parametri di regolazione del contrasto. */
  
  
  
  unsigned char	Data[2];			// command data



  Data[0]=	Volume;				// V1 value
  Data[1]=	Res;				// resistor K
  S1D15G10_SendCommand(VOLCTR,Data);
  return;					// termina
}



void PCF8833_CopyFrame(unsigned char *Frame,int FrameLen)
{
  /* Invia un frame buffer al display driver cominciando dall'angolo in alto
     a destra del display. */



  PCF8833_SetWindow(0,0,129,129);		// tutto il display
  PCF8833_SendCommand(RAMWR,Frame,FrameLen);	// riempie il display buffer
  return;					// termina
}



void PCF8833_PrintStr(char *Str)
{
  /* Stampa una stringa a video come se fosse un terminale testuale. */
}

/* $Source$ */

