/*
    S1D15G10 based LCD display driver.
    Copyright (C) 2012 2016  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __S1D15G10_C
#include "s1d15g10.h"				// functions prototypes

#include "io.h"					// I/O level
#include "spi.h"				// SPI channel
#include "font.h"				// font dei caratteri

/***************************************************************************/
/* define interni alla libreria */

  // display commands
#define SET_COLUMN_ADDRESS	0x15
#define SET_ROW_ADDRESS		0x75
#define SLEEP_OUT		0x94
#define DISPON			0xaf
#define BSTRON			0xd1
#define MEMORY_WRITE		0x5c
#define DISPLAY_NOP		0x25

#define OLED_COLS		21		// numero di colonne testo
#define OLED_ROWS		16		// numero di righe testo
#define FONT_W			6		// larghezza char
#define FONT_L			8		// altezza char

#define BYTES_LINE		64		// bytes per riga nel display
#define PIXEL_ROWS		128		// righe totali del display

#define FONT_COLOR		0x0f		// colore testo

#define INIT_DATA		0x00		// valore di inizializzazione

/***************************************************************************/
/* macro interne alla libreria */

#define SetCS(v)	IO_BitWrite(3,v)
#define SetRES(v)	IO_BitWrite(4,v)

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

//static const unsigned char	BusMap[8]={2,4,0,1,3,5,6,7};
static const unsigned char	DispSleepOut[]=	{SLEEP_OUT};
static const unsigned char	DispOn[]=	{DISPON};
static const unsigned char	DispBstrOn[]=	{BSTRON};

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

static void SetWindow(unsigned char x1,unsigned char y1,
			unsigned char x2,unsigned char y2);
  /* Imposta l'area in cui sara' aggiornata l'immagine. */

static void PutChar(int Ascii);
  /* Stampa un char alla posizione attuale del cursore, il cursore avanza
     di una posizione, raggiunta la fine della riga il cursore passa alla
     riga successiva, ragginta la fine del video riparte della prima riga. */

/***************************************************************************/
/* funzioni */

static void SetWindow(unsigned char x1,unsigned char y1,
			unsigned char x2,unsigned char y2)
{
  /* Imposta l'area in cui sara' aggiornata l'immagine. */



  unsigned char	CmdBuf[3];		// comando spedito



  // imposta le colonne della finestra
  CmdBuf[0]=0x2a;
  CmdBuf[1]=x1;
  CmdBuf[2]=x2;
  S1D15G10_CmdWrite(CmdBuf,3);		// imposta le colonne
  
  // imposta le righe della finestra
  CmdBuf[0]=0x2b;
  CmdBuf[1]=y1;
  CmdBuf[2]=y2;
  S1D15G10_CmdWrite(CmdBuf,3);		// imposta le righe
  return;				// termina
}



static void PutChar(int Ascii)
{
  /* Stampa un char alla posizione attuale del cursore, il cursore avanza
     di una posizione, raggiunta la fine della riga il cursore passa alla
     riga successiva, ragginta la fine del video riparte della prima riga. */



  const unsigned char	*Font;			// char shape
  int			i,j;
  int			k=0;
  unsigned char		Shape;			// row shape
  unsigned char		Data[3*8];		// display data array



  switch(Ascii)
  {
    case '\n':
      FontNewLine();				// va a capo
    break;
    case '\r':
      CursorX=0;				// inizio riga
    break;
    default:
      Font=FontGetChar(Ascii);			// punta alla char shape
      for(i=0;i<8;i++)				// cicla le righe
      {
        Shape=*Font;				// legge un byte dal font
	Font++;					// punta al successivo
        for(j=0;j<3;j++)			// cicla le colonne
	{
	  Shape<<=2;				// sposta avanti 2 posti
	  Data[k]=0x00;				// 2 pixel neri
	  if(Shape & 0x80)
	    Data[k]|=0xf0;
	  if(Shape & 0x40)
	    Data[k]|=0x0f;
	  k++;					// next byte in data array
	}
      }
      SetWindow(CursorX,CursorY,CursorX+2,CursorY+7);
      S1D15G10_SendData(Data,3*8);		// riempie la finestra
      CursorX+=3;
      if(CursorX > 60)
        FontNewLine();				// va a capo
    break;
  }
  return;					// termina
}



int S1D15G10_InitBus(void *Chipset,int argc,char **argv)
{
  /* Imposta lo stato iniziale del bus e inizializza il display, ritorna 0
     in caso di successo, 1 altrimenti. */
  
  
  
  // inizializza il bus
//  SPI_PinRemap(BusMap);				// rimappa i pin di I/O
  if(SPI_InitBus(SPI_CPOL_HI|SPI_SECOND_EDGE|SPI_MSB_FIRST,9))
    return 1;					// termina con fallimento
  
  // imposta direzione e stato dei pin non SPI
  IO_DirMaskSet(0x18);				// bit 3 e 4 in uscita
  SetCS(1);					// alza CS
  
  // display hardware reset
  SetRES(0);					// abbassa RES
  SetRES(1);					// alza RES

  // display power up sequence
  S1D15G10_CmdWrite(DispSleepOut,1);
  S1D15G10_CmdWrite(DispOn,1);
  S1D15G10_CmdWrite(DispBstrOn,1);

  // clear display
  SetWindow(0,0,63,127);			// tutto il display
  return 0;					// termina con successo
}



int S1D15G10_CmdWrite(const unsigned char Cmd[],int Len)
{
  /* Invia un array di command bytes al display driver. */



  unsigned long int	Command=0x0000;		// bit D/c



  SetCS(0);					// abbassa CS
  while(Len)
  {
    SPI_DataExchange(*Cmd | Command);		// invia un byte
    Cmd++;					// next byte
    Len--;					// scala il contatore
    Command=0x0100;				// seguono solo dati
  }
  SetCS(1);					// alza CS
  return 0;					// termina con successo
}



void S1D15G10_SendData(const unsigned char *Data,unsigned int DataLen)
{
  /* Invia un array di data bytes al display driver. */
  
  
  
  int			i=0;
  
  
  
  SetCS(0);					// abbassa CS
  SPI_DataExchange(MEMORY_WRITE);		// invia il comando
  for(i=0;i<DataLen;i++)
    SPI_DataExchange(Data[i]|0x0100);		// invia un data byte
  SPI_DataExchange(DISPLAY_NOP);		// termina il data stream
  SetCS(1);					// alza CS
  return;					// termina
}



int S1D15G10_CopyFrame(unsigned long int Addr,const unsigned char *Frame,int Len)
{
  /* Copia il frame buffer specificato sul diplay partendo dall'alto. */
  
  
  
  SetWindow(0,0,63,127);			// tutto il display
  S1D15G10_SendData(Frame,Len);			// copy frame buffer
  return 0;					// termina con successo
}



int S1D15G10_TextWrite(unsigned long int Addr,const char *Str,int Len)
{
  /* Stampa una stringa a video come se fosse un display alfanumerico. */
  
  
  
  while(Len--)					// fino al trminatore
    PutChar(*Str++);				// stampa un char
  return 0;					// termina con successo
}

/* $Source$ */

