/*
    SPI bus master implementation.
    Copyright (C) 2010 2011 2016 2018  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

/**  \file spi.h
     \author  Alessandro Accattini
     \date    4 Gen 2018
     \brief   SPI bus master implementation.

     SPI bus is implemented on top of I/O module: first 3 data lines (D0, D1,
     and D2) are reatined by this module and used to simulate an SPI bus
     master device.
     SPI lines are statically mapped to the hardware I/O lines, so MISO data
     line is always mapped to D0, MOSI data line is always mapped to D1, SCL
     data line is always mapped to D2.
     All other data lines are available through I/O module for other purposes;
     it is very important to not overwrite first 3 data lines settings while
     SPI bus is in use.
 */


#ifndef __SPI_H
#define __SPI_H	1


/***************************************************************************/
/* define pubblici */

  // modalita' di funzionamento dell'SPI
#define SPI_CPOL_LOW		0x00
#define SPI_CPOL_HI		0x01
#define SPI_FIRST_EDGE		0x00
#define SPI_SECOND_EDGE		0x02
#define SPI_LSB_FIRST		0x00
#define SPI_MSB_FIRST		0x04

/***************************************************************************/
/* macro pubbliche */

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* costanti pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

int SPI_InitBus(int Mode,int BitPerWord);
  /**< \brief  Set the initial state of SPI bus data lines.
       \param  Mode  bus mode.
       \param  BitPerWord  SPI bus word size.
       \return returns 0 on success.

       Sets the direction and initial state of data lines used to implement
       the SPI bus; after this call first 3 I/O data lines are under control
       of this module and its status should not be over written calling I/O
       module functions.
   */

unsigned long int SPI_DataExchange(unsigned long int Data);
  /**< \brief  SPI bus data exchange.
       \param  Data  data sent over the SPI bus.
       \return data received over the SPI bus.

       This function is used to exchange data over the SPI bus. SPI bus is
       bidirectional: while a word is transmitted from master to slave,
       another word is transmitted back from slave to master.
   */

void SPI_DataWrite(unsigned long int Data);
  /* Invia una word sul bus SPI. */
  /**< \brief  SPI bus data write.
       \param  Data  data sent over the SPI bus.

       This function is the same as SPI_DataExchange() except that the
       received word from slave device is always discarted.
       SPI bus is emulated over I/O data lines, this function may be faster
       than SPI_DataExchange() when slave reply is not required.
   */

/***************************************************************************/
#endif

/* $Source$ */

