/*
    SSD1328 based oled display driver.
    Copyright (C) 2008 2011 2012 2016  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __SSD1328_C
#include "ssd1328.h"				// functions prototypes

#include "io.h"					// I/O level
#include "spi.h"				// SPI channel
#include "font.h"				// font dei caratteri

/***************************************************************************/
/* define interni alla libreria */

//#define BUFFER_SIZE	64*128			// dimensione frame buffer

// display commands
#define SET_COLUMN_ADDRESS        0x15
#define SET_ROW_ADDRESS           0x75
#define SET_CONTRAST_CONTROL_REG  0x81
#define SET_CURRENT_RANGE_Q       0x84
#define SET_CURRENT_RANGE_H       0x85
#define SET_CURRENT_RANGE_F       0x86
#define SET_RE_MAP                0xA0
#define SET_DISPLAY_START_LINE    0xA1
#define SET_DISPLAY_OFFSET        0xA2
#define SET_DISPLAY_MODE_NORM     0xA4
#define SET_DISPLAY_MODE_ALL_ON   0xA5
#define SET_DISPLAY_MODE_ALL_OFF  0xA6
#define SET_DISPLAY_MODE_INV      0xA7
#define SET_MULTIPLEX_RATIO       0xA8
#define INITIALIZATION            0xAD
#define DISPLAY_ON                0xAF
#define DISPLAY_OFF               0xAE
#define OPTIMIZE_ROW_BRIGHTNESS   0xB0
#define DISPLAY_NOP               0xE3

#define OLED_COLS		21		// numero di colonne testo
#define OLED_ROWS		16		// numero di righe testo
#define FONT_W			6		// larghezza char
#define FONT_L			8		// altezza char

#define BYTES_LINE		64		// bytes per riga nel display
#define PIXEL_ROWS		128		// righe totali del display

#define FONT_COLOR		0x0f		// colore testo

#define INIT_DATA		0x00		// valore di inizializzazione

/***************************************************************************/
/* macro interne alla libreria */

#define SetCS(v)	IO_BitWrite(3,v)
#define SetRES(v)	IO_BitWrite(4,v)
#define SetNEN(v)	IO_BitWrite(5,v)
#define SetSEL(v)	IO_BitWrite(6,v)
#define SetDC(v)	IO_BitWrite(7,v)

#define SetLamp(v)	SetNEN(v)

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

//static const unsigned char	BusMap[8]={0,7,2,3,6,5,4,1};
static const unsigned char	DispInit[]=	{INITIALIZATION,0x02};
static const unsigned char	DispOn[]=	{DISPLAY_ON};
static const unsigned char	DispRemap[]=	{SET_RE_MAP,0x52};

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

static void SetWindow(unsigned char x1,unsigned char y1,
			unsigned char x2,unsigned char y2);
  /* Imposta l'area in cui sara' aggiornata l'immagine. */

static void PutChar(int Ascii);
  /* Stampa un char alla posizione attuale del cursore, il cursore avanza
     di una posizione, raggiunta la fine della riga il cursore passa alla
     riga successiva, ragginta la fine del video riparte della prima riga. */

/***************************************************************************/
/* funzioni */

static void SetWindow(unsigned char x1,unsigned char y1,
			unsigned char x2,unsigned char y2)
{
  /* Imposta l'area in cui sara' aggiornata l'immagine. */



  unsigned char	CmdBuf[3];		// comando spedito



  // imposta le colonne della finestra
  CmdBuf[0]=0x15;
  CmdBuf[1]=x1;
  CmdBuf[2]=x2;
  SSD1328_CmdWrite(CmdBuf,3);		// imposta le colonne
  
  // imposta le righe della finestra
  CmdBuf[0]=0x75;
  CmdBuf[1]=y1;
  CmdBuf[2]=y2;
  SSD1328_CmdWrite(CmdBuf,3);		// imposta le righe
  return;				// termina
}



static void PutChar(int Ascii)
{
  /* Stampa un char alla posizione attuale del cursore, il cursore avanza
     di una posizione, raggiunta la fine della riga il cursore passa alla
     riga successiva, ragginta la fine del video riparte della prima riga. */



  const unsigned char	*Font;			// char shape
  int			i,j;
  int			k=0;
  unsigned char		Shape;			// row shape
  unsigned char		Data[3*8];		// display data array



  switch(Ascii)
  {
    case '\n':
      FontNewLine();				// va a capo
    break;
    case '\r':
      CursorX=0;				// inizio riga
    break;
    default:
      Font=FontGetChar(Ascii);			// punta alla char shape
      for(i=0;i<8;i++)				// cicla le righe
      {
        Shape=*Font;				// legge un byte dal font
	Font++;					// punta al successivo
        for(j=0;j<3;j++)			// cicla le colonne
	{
	  Shape<<=2;				// sposta avanti 2 posti
	  Data[k]=0x00;				// 2 pixel neri
	  if(Shape & 0x80)
	    Data[k]|=0xf0;
	  if(Shape & 0x40)
	    Data[k]|=0x0f;
	  k++;					// next byte in data array
	}
      }
      SetWindow(CursorX,CursorY,CursorX+2,CursorY+7);
      SSD1328_SendData(Data,3*8);		// riempie la finestra
      CursorX+=3;
      if(CursorX > 60)
        FontNewLine();				// va a capo
    break;
  }
  return;					// termina
}



int SSD1328_InitBus(void *Chipset,int argc,char **argv)
{
  /* Imposta lo stato iniziale del bus e inizializza il display, ritorna 0
     in caso di successo, 1 altrimenti. */
  
  
  
  // inizializza il bus
  //SPI_PinRemap(BusMap);				// rimappa i pin di I/O
  if(SPI_InitBus(SPI_CPOL_HI|SPI_SECOND_EDGE|SPI_MSB_FIRST,8))
    return 1;					// termina con fallimento

  // imposta direzione e stato dei pin non SPI
  IO_DirMaskSet(0xf8);				// tutti e 5 in uscita
  SetCS(1);					// alza CS
  SetDC(1);					// data mode
  SetLamp(1);					// accende la lampada
  SetSEL(0);					// lampada al massimo
  
  // display hardware reset
  SetRES(0);					// abbassa RES
  SetRES(1);					// alza RES

  // display power up sequence
  SSD1328_CmdWrite(DispInit,2);
  SSD1328_CmdWrite(DispOn,1);
  SSD1328_CmdWrite(DispRemap,2);

  // clear display
  SetWindow(0,0,63,127);			// tutto il display
  return 0;					// termina con successo
}



int SSD1328_CmdWrite(const unsigned char Cmd[],int Len)
{
  /* Invia un array di command bytes al display driver. */



  SetDC(0);					// command mode
  SetCS(0);					// abbassa CS
  while(Len)
  {
    SPI_DataExchange(*Cmd);			// invia un command byte
    Cmd++;					// next byte
    Len--;					// scala il contatore
  }
  SetCS(1);					// alza CS
  SetDC(1);					// data mode
  return 0;					// termina con successo
}



void SSD1328_SendData(const unsigned char *Data,unsigned int DataLen)
{
  /* Invia un array di data bytes al display driver. */
  
  
  
  int			i=0;
  
  
  
  SetCS(0);					// abbassa CS
  for(i=0;i<DataLen;i++)
    SPI_DataExchange(Data[i]);			// invia un data byte
  SetDC(0);					// command mode
  SPI_DataExchange(DISPLAY_NOP);		// termina il data stream
  SetCS(1);					// alza CS
  SetDC(1);					// data mode
  return;					// termina
}



int SSD1328_CopyFrame(unsigned long int Addr,const unsigned char *Frame,int Len)
{
  /* Copia il frame buffer specificato sul diplay partendo dall'alto. */
  
  
  
  SetWindow(0,0,63,127);			// tutto il display
  SSD1328_SendData(Frame,Len);			// copy frame buffer
  return 0;					// termina con successo
}



int SSD1328_TextWrite(unsigned long int Addr,const char *Str,int Len)
{
  /* Stampa una stringa a video come se fosse un display alfanumerico. */
  
  
  
  while(Len--)					// fino al trminatore
    PutChar(*Str++);				// stampa un char
  return 0;					// termina con successo
}

/* $Source$ */

