/*
    Display control device implementation.
    Copyright (C) 2008  Alessandro Accattini


 *****************************************************************************
 *
 *
 * $Header: /users/CVS/oled_linux/ctrldev.c,v 1.1.1.1 2009/10/01 14:49:33 akerue Exp $
 *
 *
 *
 * REVISION HISTORY:
 * $Log: ctrldev.c,v $
 * Revision 1.1.1.1  2009/10/01 14:49:33  akerue
 * oled linux driver
 *
 */

static const char rcsid[]="$Id: ctrldev.c,v 1.1.1.1 2009/10/01 14:49:33 akerue Exp $";


/***************************************************************************/
/* include vari */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <asm/semaphore.h>			// semaphores
#include <asm/uaccess.h>			// user space access

#include "debug.h"				// debug and print macros
#include "oledf.h"				// OLED control
#include "led.h"				// LED control
#include "io.h"					// hardware control

/***************************************************************************/
/* define interni alla libreria */

#define CMDBUFFER_SIZE	64			// dimensione buffer comandi

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* variabili interne alla libreria */

static unsigned char	CtrlBuffer[CMDBUFFER_SIZE];	// ctrl queue
static unsigned char	Keys=0;			// keys status
static int		Green=0;		// stato led verde

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* define pubblici */

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

ssize_t CtrlRead(struct file *Filp,char __user *Buff,
			size_t Count,loff_t *Offp);
  /* Read method, read data from the device. */

ssize_t CtrlWrite(struct file *Filp,const char __user *Buff,
			size_t Count,loff_t *Offp);
  /* Write method, write data to the device. */

loff_t CtrlSeek(struct file *Filp,loff_t Off,int Whence);
  /* Seek method, riposiziona il puntatore R/W al file. */

/***************************************************************************/
/* funzioni */

ssize_t CtrlRead(struct file *Filp,char __user *Buff,
			size_t Count,loff_t *Offp)
{
  /* Read method, read data from the device. */


  

  /* debug info */
  PDEBUG("%s(Count=%li,Off=%li)\n",__FUNCTION__,
		(unsigned long int)Count,(unsigned long int)*Offp);
  
  /* determina il numero massimo di bytes leggibili dal device */
  if((unsigned long int)*Offp>=1)
  {
    PDEBUG("%s: no read at all, return 0\n",__FUNCTION__);
    return 0;				// termina senza lettura
  }
  if(Count>1)
    Count=1;				// limita la lettura ad un solo byte

  /* effettua la lettura dal device */
  while(down_interruptible(&HardSem));	// blocca il semaforo
  Keys=(unsigned char)(Get_KEY1()&0x01);	// legge il tasto 1
  Keys|=((unsigned char)(Get_KEY2()&0x01))<<1;	// legge il tasto 2
  Keys|=((unsigned char)(Get_KEY3()&0x01))<<2;	// legge il tasto 3
  Keys|=((unsigned char)(Get_SUSPEND()&0x01))<<3;	// legge SUSPEND
  Keys^=0x07;				// inverte i bit dei tasti
  if(copy_to_user(Buff,&Keys,(unsigned long int)Count))
  {
    up(&HardSem);			// libera il semaforo
    return(-EFAULT);			// segmentation fault
  }

  /* aggiorna il puntatore R/W al file */
  *Offp=0;				// forza il puntatore a 0
  up(&HardSem);				// libera il semaforo
  return((ssize_t)Count);		// termina con successo
}



ssize_t CtrlWrite(struct file *Filp,const char __user *Buff,
			size_t Count,loff_t *Offp)
{
  /* Write method, write data to the device. */
  
  
  
  unsigned long int	MaxWriteBytes;
  unsigned long int	i;



  /* debug info */
  PDEBUG("%s(Count=%li,Off=%li)\n",__FUNCTION__,
		(unsigned long int)Count,(unsigned long int)*Offp);
  
  /* determina il numero massimo di bytes scrivibili sul device */
  if(*Offp>=CMDBUFFER_SIZE)
  {
    PDEBUG("%s: no write at all, return 0\n",__FUNCTION__);
    return 0;				// termina senza lettura
  }
  MaxWriteBytes=CMDBUFFER_SIZE-(unsigned long int)*Offp;
  if(Count>MaxWriteBytes)
    Count=MaxWriteBytes;		// limita la scrittura

  /* effettua la scrittura sul device */
  while(down_interruptible(&HardSem));	// blocca il semaforo
  if(copy_from_user(&CtrlBuffer[*Offp],Buff,(unsigned long int)Count))
  {
    up(&HardSem);			// libera il semaforo
    return(-EFAULT);			// segmentation fault
  }
  *Offp=0;				// forza il puntatore a zero

  /* interpreta i comandi 1 byte per comando */
  for(i=0;i<(unsigned long int)Count;++i)
  {
    switch(CtrlBuffer[i])
    {
      case 'e':
	OledTurnOffDisplay();		// disabilita il display
      break;
      case 'E':
	OledTurnOnDisplay();		// abilita il display
      break;
      case 'l':
      case 'r':
        Set_LED1(0);			// spegne il LED1
      break;
      case 'L':
      case 'R':
        Set_LED1(1);			// accende il LED1
      break;
      case 'o':
        Set_LED2(0);			// spegne il LED2
      break;
      case 'O':
        Set_LED2(1);			// accende il LED2
      break;
      case 'g':
        Green=0;			// spegne il LED3
        LedGreen(Green);
      break;
      case 'G':
	Green=1;			// accende il LED3
        LedGreen(Green);
      break;
      case 'T':
      case 't':
	Green^=1;			// toggle del LED3
        LedGreen(Green);
      break;
    }
  }
  up(&HardSem);				// libera il semaforo
  return((ssize_t)Count);		// termina con successo
}



loff_t CtrlSeek(struct file *Filp,loff_t Off,int Whence)
{
  /* Seek method, riposiziona il puntatore R/W al file. */



  Filp->f_pos=0;			// blocca lo scorrimento del file
  return(-EINVAL);			// seek invalid termina con errore
}

/* $Source: /users/CVS/oled_linux/ctrldev.c,v $ */

