/*
    Data device accessing functions.
    Copyright (C) 2008  Alessandro Accattini


 *****************************************************************************
 *
 *
 *
 */



/***************************************************************************/
/* include vari */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <asm/semaphore.h>			// semaphores
#include <asm/uaccess.h>			// user space access

#include "debug.h"				// debug and print macros
#include "oledf.h"				// display control
//#include "io.h"					// hardware control

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* define pubblici */

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

ssize_t DataRead(struct file *Filp,char __user *Buff,
			size_t Count,loff_t *Offp);
  /* Read method, read data from the device. */

ssize_t DataWrite(struct file *Filp,const char __user *Buff,
			size_t Count,loff_t *Offp);
  /* Write method, write data to the device. */

loff_t DataSeek(struct file *Filp,loff_t Off,int Whence);
  /* Seek method, riposiziona il puntatore R/W al file. */

/***************************************************************************/
/* funzioni */

ssize_t DataRead(struct file *Filp,char __user *Buff,
			size_t Count,loff_t *Offp)
{
  /* Read method, read data from the device. */



  unsigned long int	MaxReadBytes;



  /* debug info */
  PDEBUG("%s(Count=%li,Off=%li)\n",__FUNCTION__,
		(unsigned long int)Count,(unsigned long int)*Offp);
  
  /* determina il numero massimo di bytes leggibili dal device */
  if(*Offp>=BUFFER_SIZE)
  {
    PDEBUG("%s: no read at all, return 0\n",__FUNCTION__);
    return 0;				// termina senza lettura
  }
  MaxReadBytes=BUFFER_SIZE-(unsigned long int)*Offp;
  if(Count>MaxReadBytes)
    Count=MaxReadBytes;			// limita la lettura

  /* effettua la lettura dal device */
  while(down_interruptible(&HardSem));	// blocca il semaforo
  if(copy_to_user(Buff,&DataBuffer[*Offp],(unsigned long int)Count))
  {
    up(&HardSem);			// libera il semaforo
    return(-EFAULT);			// segmentation fault
  }

  /* aggiorna il puntatore R/W al file */
  *Offp+=Count;				// incrementa il puntatore
  up(&HardSem);				// libera il semaforo
  return((ssize_t)Count);		// termina con successo
}



ssize_t DataWrite(struct file *Filp,const char __user *Buff,
			size_t Count,loff_t *Offp)
{
  /* Write method, write data to the device. */
  
  
  
  unsigned long int	MaxWriteBytes;



  /* debug info */
  PDEBUG("%s(Count=%li,Off=%li)\n",__FUNCTION__,
		(unsigned long int)Count,(unsigned long int)*Offp);
  
  /* determina il numero massimo di bytes scrivibili sul device */
  if(*Offp>=BUFFER_SIZE)
  {
    PDEBUG("%s: no write at all, return 0\n",__FUNCTION__);
    return 0;				// termina senza lettura
  }
  MaxWriteBytes=BUFFER_SIZE-(unsigned long int)*Offp;
  if(Count>MaxWriteBytes)
    Count=MaxWriteBytes;		// limita la scrittura

  /* effettua la scrittura sul device */
  while(down_interruptible(&HardSem));	// blocca il semaforo
  if(copy_from_user(&DataBuffer[*Offp],Buff,(unsigned long int)Count))
  {
    up(&HardSem);			// libera il semaforo
    return(-EFAULT);			// segmentation fault
  }
  OledTurnOnDisplay();			// abilita il display
  OledRefreshImg();			// rinfresca l'intera immagine
  *Offp+=Count;				// incrementa il puntatore
  up(&HardSem);				// libera il semaforo
  return((ssize_t)Count);		// termina con successo
}



loff_t DataSeek(struct file *Filp,loff_t Off,int Whence)
{
  /* Seek method, riposiziona il puntatore R/W al file. */



  if(Whence!=0)
    return(-EINVAL);			// termina con errore

  /* riposiziona il puntatore in modo assoluto dall'inizio del file */
  if(Off>BUFFER_SIZE-1)
    Off=BUFFER_SIZE-1;			// limita la posizione del puntatore
  Filp->f_pos=Off;			// riposiziona il puntatore
  return(Off);				// termina con successo
}

/* $Source: /users/CVS/oled_linux/datadev.c,v $ */

