/*
    Universal hardware control environment.
    Copyright (C) 2010 2011 2012 2013 2014 2016  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <time.h>			// execution time measurement
#include "commands/commands.h"		// commands lib

/***************************************************************************/
/* define vari */

#define PDEBUG(fmt,args...)	printf(fmt,## args)

  // flag vari
#define FLAG_EXEC_TIME		0x01
#define FLAG_INTERACTIVE	0x02
#define FLAG_COMMAND_PRINT	0x04

/***************************************************************************/
/* macro */

/***************************************************************************/
/* strutture */

/***************************************************************************/
/* costanti */

static const char Licenza[]=
	"struttura version 0.11, Copyright (C) 2010-2016  "
	"Alessandro Accattini\n";
static const char Help[]=
	"Usage: struttura [OPTIONS] [file_1] [file_2]...\n"
	"struttura is an environment to drive hardware devices.\n\n"
	"\t-h\tdisplay this help.\n"
	"\t-l\tlist all supported chipsets and exit.\n"
	"\t-L\tlong list of all supported chipsets.\n"
	"\t-v N\tset verbosity level.\n"
	"\t-C\tprints out the command to being executed.\n"
	"\t-I N\tforce N hardware interfaces.\n"
	"\t-P HEX\tset chipset product code.\n"
	"\t-T\tenable commands execution time measure.\n"
	"\t-V HEX\tset chipset vendor code.\n"
	"\nReport bugs to bit.trapper@gmail.com\n";

static char	Prompt[]="struttura> ";	// prompt dei comandi

/***************************************************************************/
/* variabili */

char	InBuf[256];			// input da tastiera

/***************************************************************************/
/* prototipi funzioni */

/***************************************************************************/
/* funzioni */





/***************************************************************************/

int main(int argc,char **argv)
{
  time_t	Tic,Toc;		// usati per cronometrare i comandi
  int		Vendor=0x0403;		// vendor code
  int		Product=0x6001;		// product code
  int		Opzione;		// per la lettura delle opzioni
  int		HardIfaces=0;		// numero interfacce hardware
  unsigned int	Mode=0;			// flag vari



  // visualizza la licenza del programma
  puts(Licenza);
  
  // legge le opzioni presenti sulla riga di comando
  while((Opzione=getopt(argc,argv,"hlLTv:CV:P:I:"))!=-1)
  {
    switch(Opzione)
    {
      case 'h':
        puts(Help);			// stampa l'help
	return 0;			// termina
      case 'l':
        sprintf(InBuf,"list");
        ExecCmd(InBuf);			// stampa la lista dei chipset
	return 0;			// termina
      case 'L':
        sprintf(InBuf,"list long");
        ExecCmd(InBuf);			// stampa la lista dei chipset
	return 0;			// termina
      case 'v':
        sscanf(optarg,"%i",&Verbosity);	// imposta la verbosita'
	break;
      case 'V':
        sscanf(optarg,"%i",&Vendor);	// imposta il vendor code
	break;
      case 'P':
        sscanf(optarg,"%i",&Product);	// imposta il product code
	break;
      case 'I':
        sscanf(optarg,"%i",&HardIfaces);	// forza le interfacce hardware
	break;
      case 'C':
        Mode|=FLAG_COMMAND_PRINT;	// abilita stampa dei comandi eseguiti
	break;
      case 'T':
        Mode|=FLAG_EXEC_TIME;		// abilita tempo esecuzione comandi
	break;
      default:
        puts("Unknown option, use -h to see help.");
	return 0;			// termina
    }
  }

  // apre il controllo dell'interfacia hardware
  if(IO_Open(Vendor,Product,HardIfaces))	// se fallisce
    return 1;				// termina con fallimento
  putchar('\n');			// lascia una riga

  // processa tutti gli altri parametri passati
  if(optind >= argc)			// se non ci sono file sorgenti
    Mode|=FLAG_INTERACTIVE;		// passa in modalita' interattiva
  do  
  {
    if(!(Mode & FLAG_INTERACTIVE))	// se non interattivo
    {
      if(freopen(argv[optind++],"r",stdin) == NULL)	// sostituisce lo stdin
      {
	perror("freopen");		// visualizza l'errore
	continue;			// passa al prossimo file
      }
    }
  
    // interpreta il file sorgente
    while(1)
    {
      // legge un comando dallo stdin
      if(Mode & FLAG_INTERACTIVE)	// se interattivo
	printf(Prompt);			// stampa il prompt dei comandi
      if(fgets(InBuf,256,stdin)==NULL)	// legge una stringa da tastiera
	break;				// abbandona il main loop
      if(Mode & FLAG_COMMAND_PRINT)	// se richiesto
	printf(InBuf);			// stampa la riga comandi

      // esegue il comando
      time(&Tic);			// istante iniziale
      if(ExecCmd(InBuf))		// esegue il comando digitatato
	break;				// abbandona il main loop
      if(Mode & FLAG_EXEC_TIME)		// se abilitata la misura del tempo
      {
	time(&Toc);			// istante finale
	printf(" execution time: %4.2lfs\n",difftime(Toc,Tic));
      }
    }
  }while(optind < argc);

  IO_Close();				// chiude l'hardware
  fclose(stdin);			// chiude l'input stream
  puts("\n\nThank you for using 'struttura'!\n");
  return 0;				// termina
}

/* $Source$ */

